@@ -70,6 +70,7 @@ typedef enum {
 typedef struct {
 	GthFileListOpType type;
 	union {
+		GList *fd_list;
 		GList *uri_list;
 		char  *uri;
 		int    ival;
@@ -87,6 +88,7 @@ struct _GthFileListPrivateData {
 
 	gboolean       show_dot_files;      /* Whether to show files that starts
 					     * with a dot (hidden files).*/
+	gboolean       ignore_hidden_thumbs;
 	gboolean       load_thumbs;
 
 	int            thumb_size;          /* Thumbnails max size. */
@@ -134,7 +136,7 @@ static void gth_file_list_exec_next_op (GthFileList *file_list);
 
 
 static GthFileListOp *
-gth_file_list_op_new (op_type)
+gth_file_list_op_new (GthFileListOpType op_type)
 {
 	GthFileListOp *op;
 
@@ -155,9 +157,11 @@ gth_file_list_op_free (GthFileListOp *op)
 		break;
 	case GTH_FILE_LIST_OP_TYPE_RENAME:
 	case GTH_FILE_LIST_OP_TYPE_UPDATE_THUMB_LIST:
+		path_list_free (op->file.uri_list);
+		break;
 	case GTH_FILE_LIST_OP_TYPE_ADD_LIST:
 	case GTH_FILE_LIST_OP_TYPE_SET_LIST:
-		path_list_free (op->file.uri_list);
+		file_data_list_free (op->file.fd_list);
 		break;
 	default:
 		break;
@@ -228,6 +232,20 @@ gth_file_list_queue_op_with_uri (GthFileList       *file_list,
 }
 
 
+static void
+gth_file_list_queue_op_with_file_data_list (GthFileList       *file_list,
+				  	    GthFileListOpType  op_type,
+				  	    GList             *fd_list)
+{
+	GthFileListOp *op;
+
+	op = gth_file_list_op_new (op_type);
+	op->file.fd_list = file_data_list_dup (fd_list);
+
+	gth_file_list_queue_op (file_list, op);
+}
+
+
 static void
 gth_file_list_queue_op_with_list (GthFileList       *file_list,
 				  GthFileListOpType  op_type,
@@ -298,9 +316,6 @@ update_thumbs_stopped (gpointer callback_data)
 static void
 gth_file_list_update_current_thumb (GthFileList *file_list)
 {
-	gboolean  error = TRUE;
-	char     *path;
-
 	if (file_list->priv->stop_it || (file_list->priv->queue != NULL)) {
 		g_idle_add (update_thumbs_stopped, file_list);
 		return;
@@ -309,32 +324,8 @@ gth_file_list_update_current_thumb (GthFileList *file_list)
 	g_return_if_fail (file_list->priv->thumb_fd != NULL);
 
 	file_list->priv->loading_thumbs = TRUE;
-
-	path = g_strdup (file_list->priv->thumb_fd->path);
-	if (path_is_file (path)) {
-		GnomeVFSResult  result;
-		char           *resolved_path = NULL;
-
-		result = resolve_all_symlinks (path, &resolved_path);
-
-		if (result == GNOME_VFS_OK) {
-			if (resolved_path != NULL) {
-				thumb_loader_set_path (file_list->priv->thumb_loader, resolved_path, file_list->priv->thumb_fd->mime_type);
-				thumb_loader_start (file_list->priv->thumb_loader);
-				error = FALSE;
-			}
-		} else
-			g_warning ("%s\n", gnome_vfs_result_to_string (result));
-
-		g_free (resolved_path);
-	}
-	g_free (path);
-
-	if (error) /* Error: the file does not exists. */
-		g_signal_emit_by_name (G_OBJECT (file_list->priv->thumb_loader),
-				       "thumb_error",
-				       0,
-				       file_list);
+	thumb_loader_set_file (file_list->priv->thumb_loader, file_list->priv->thumb_fd);
+	thumb_loader_start (file_list->priv->thumb_loader);
 }
 
 
@@ -376,73 +367,82 @@ gth_file_list_update_next_thumb (GthFileList *file_list)
 	list = gth_file_view_get_list (file_list->view);
 	scan = g_list_nth (list, pos);
 	if (scan == NULL) {
+		file_data_list_free (list);
 		gth_file_list_done (file_list);
 		return;
 	}
 
 	/* Find a not loaded thumbnail among the visible images. */
+	
 	while (pos <= last_pos) {
 		fd = scan->data;
 		if (! fd->thumb_loaded && ! fd->error) {
 			new_pos = pos;
 			break;
-		} else {
+		} 
+		else {
 			pos++;
 			scan = scan->next;
 		}
 	}
 
-	/* Find a not created thumbnail among the not-visible images. */
+	if (! file_list->priv->ignore_hidden_thumbs) {
 
-	/* start from after the last visible image... */
+		/* Find a not created thumbnail among the not-visible images. */
 
-	if (new_pos == -1) {
-		pos = last_pos + 1;
-		scan = g_list_nth (list, pos);
-		while (scan) {
-			fd = scan->data;
-			if (! fd->thumb_created && ! fd->error) {
-				new_pos = pos;
-				break;
+		/* start from the one after the last visible image... */
+
+		if (new_pos == -1) {
+			pos = last_pos + 1;
+			scan = g_list_nth (list, pos);
+			while (scan) {
+				fd = scan->data;
+				if (! fd->thumb_created && ! fd->error) {
+					new_pos = pos;
+					break;
+				}
+				pos++;
+				scan = scan->next;
 			}
-			pos++;
-			scan = scan->next;
 		}
-	}
 
-	/* ...continue from the beginning */
+		/* ...continue from the one before the first visible upward to 
+		 * the first one */
 
-	if (new_pos == -1) {
-		pos = 0;
-		scan = list;
-		while (scan && (pos < first_pos)) {
-			fd = scan->data;
-			if (! fd->thumb_created && ! fd->error) {
-				new_pos = pos;
-				break;
+		if (new_pos == -1) {
+			pos = first_pos - 1;
+			scan = g_list_nth (list, pos);
+			while (scan) {
+				fd = scan->data;
+				if (! fd->thumb_created && ! fd->error) {
+					new_pos = pos;
+					break;
+				}
+				pos--;
+				scan = scan->prev;
 			}
-			pos++;
-			scan = scan->next;
 		}
 	}
 
-	g_list_free (list);
+	if (new_pos != -1)
+		fd = file_data_ref (fd);
+
+	file_data_list_free (list);
 
 	if (new_pos == -1) {
 		gth_file_list_done (file_list);
 		return;
 	}
 
-	g_assert (fd != NULL);
-
-	file_list->priv->update_thumb_in_image_list = (new_pos >= first_pos) && (new_pos <= last_pos);
+	/* We create thumbnail files for all images in the folder, but we only
+	   load the visible ones (and 25 before and 25 after the visible range),
+	   to minimize memory consumption in large folders. */
+	file_list->priv->update_thumb_in_image_list = (new_pos >= (first_pos - 25)) &&
+						      (new_pos <= (last_pos + 25));
 	file_list->priv->thumb_pos = new_pos;
 	file_list->priv->thumbs_num++;
-
-	if (file_list->priv->thumb_fd != NULL)
-		file_data_unref (file_list->priv->thumb_fd);
-	file_list->priv->thumb_fd = fd;
-	file_data_ref (file_list->priv->thumb_fd);
+	file_data_unref (file_list->priv->thumb_fd);
+	file_list->priv->thumb_fd = fd; /* already ref-ed above */
 
 	gth_file_list_update_current_thumb (file_list);
 }
@@ -496,6 +496,22 @@ gth_file_list_update_icon_theme (GthFileList *file_list)
 }
 
 
+void         
+gth_file_list_show_hidden_files (GthFileList *file_list,
+				 gboolean     show)
+{
+	file_list->priv->show_dot_files = show;
+}
+
+
+void
+gth_file_list_ignore_hidden_thumbs (GthFileList *file_list,
+				    gboolean     ignore)
+{
+	file_list->priv->ignore_hidden_thumbs = ignore;
+}
+
+
 static int
 get_default_icon_size (GtkWidget *widget)
 {
@@ -607,11 +623,12 @@ gth_file_list_update_thumbs (GthFileList *file_list)
 	int    i;
 	GList *scan;
 
+	thumb_loader_save_thumbnails (THUMB_LOADER (file_list->priv->thumb_loader), eel_gconf_get_boolean (PREF_SAVE_THUMBNAILS, TRUE));
+	thumb_loader_set_max_file_size (THUMB_LOADER (file_list->priv->thumb_loader), eel_gconf_get_integer (PREF_THUMBNAIL_LIMIT, 0));
+
 	for (i = 0; i < gth_file_view_get_images (file_list->view); i++)
 		set_unknown_pixbuf (file_list, i);
 
-	thumb_loader_set_max_file_size (THUMB_LOADER (file_list->priv->thumb_loader), eel_gconf_get_integer (PREF_THUMBNAIL_LIMIT, 0));
-
 	for (scan = file_list->list; scan; scan = scan->next) {
 		FileData *fd = scan->data;
 		fd->thumb_loaded = FALSE;
@@ -719,7 +736,7 @@ gth_file_list_free_list (GthFileList *file_list)
 {
 	g_return_if_fail (file_list != NULL);
 
-	path_list_free (file_list->priv->new_list);
+	file_data_list_free (file_list->priv->new_list);
 	file_list->priv->new_list = NULL;
 
 	file_data_list_free (file_list->list);
@@ -849,10 +866,10 @@ gth_file_list_init (GthFileList *file_list)
 	file_list->list                 = NULL;
 	file_list->priv->sort_method    = pref_get_arrange_type ();
 	file_list->priv->sort_type      = pref_get_sort_order ();
-	file_list->priv->show_dot_files = eel_gconf_get_boolean (PREF_SHOW_HIDDEN_FILES, FALSE);
 	file_list->enable_thumbs        = eel_gconf_get_boolean (PREF_SHOW_THUMBNAILS, TRUE);
+	file_list->priv->show_dot_files = eel_gconf_get_boolean (PREF_SHOW_HIDDEN_FILES, FALSE);	
 	file_list->priv->thumb_size     = eel_gconf_get_integer (PREF_THUMBNAIL_SIZE, DEF_THUMB_SIZE);
-	file_list->priv->thumb_loader   = THUMB_LOADER (thumb_loader_new (NULL, file_list->priv->thumb_size, file_list->priv->thumb_size));
+	file_list->priv->thumb_loader   = THUMB_LOADER (thumb_loader_new (file_list->priv->thumb_size, file_list->priv->thumb_size));
 	file_list->priv->thumbs_num     = 0;
 	file_list->priv->thumb_fd       = NULL;
 	file_list->priv->thumb_pos      = -1;
@@ -862,7 +879,8 @@ gth_file_list_init (GthFileList *file_list)
 	file_list->priv->loading_thumbs = FALSE;
 	file_list->priv->filter         = NULL;
 	file_list->priv->pixbufs        = g_hash_table_new (g_str_hash, g_str_equal);
-
+	file_list->priv->ignore_hidden_thumbs = FALSE;
+	
 	g_signal_connect (G_OBJECT (file_list->priv->thumb_loader),
 			  "thumb_done",
 			  G_CALLBACK (load_thumb_done_cb),
@@ -968,58 +986,44 @@ gth_file_list_get_view (GthFileList *file_list)
 }
 
 
-/* -- GetFileInfoData -- */
+/* -- AddListData -- */
 
 
 typedef struct {
 	GthFileList *file_list;
 	GList       *new_list;
-	GList       *filtered;
-	GList       *uri_list;
-	GHashTable  *mime_types;
+	GList       *scan;
 	guint        timeout_id;
 	gboolean     doing_thumbs;
 	gboolean     destroyed;
-} GetFileInfoData;
+} AddListData;
 
 
-static GetFileInfoData *
-get_file_info_data_new (GthFileList *file_list)
+static AddListData *
+add_list_data_new (GthFileList *file_list)
 {
-	GetFileInfoData * data;
+	AddListData * data;
 
-	data = g_new0 (GetFileInfoData, 1);
+	data = g_new0 (AddListData, 1);
 	data->file_list = file_list;
 	data->new_list = file_list->priv->new_list;
-	file_list->priv->new_list = NULL;
-	data->filtered = NULL;
-	data->uri_list = NULL;
-	data->mime_types = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+	data->scan = data->new_list;
 	data->timeout_id = 0;
 	data->destroyed = FALSE;
 
+	file_list->priv->new_list = NULL;
+
 	return data;
 }
 
 
 static void
-get_file_info_data_free (GetFileInfoData *data)
+add_list_data_free (AddListData *data)
 {
 	if (data == NULL)
 		return;
 
-	g_hash_table_destroy (data->mime_types);
-
-	if (data->uri_list != NULL) {
-		g_list_foreach (data->uri_list,
-				(GFunc) gnome_vfs_uri_unref,
-				NULL);
-		g_list_free (data->uri_list);
-	}
-
-	path_list_free (data->new_list);
-	file_data_list_free (data->filtered);
-
+	file_data_list_free (data->new_list);
 	g_free (data);
 }
 
@@ -1030,10 +1034,10 @@ get_file_info_data_free (GetFileInfoData *data)
 static gboolean
 add_list_in_chunks_stopped (gpointer callback_data)
 {
-	GetFileInfoData *gfi_data = callback_data;
-	GthFileList     *file_list = gfi_data->file_list;
+	AddListData *al_data = callback_data;
+	GthFileList *file_list = al_data->file_list;
 
-	get_file_info_data_free (gfi_data);
+	add_list_data_free (al_data);
 	gth_file_list_exec_next_op (file_list);
 
 	return FALSE;
@@ -1061,32 +1065,32 @@ add_list_done (GthFileList *file_list)
 static gboolean
 add_list_in_chunks (gpointer callback_data)
 {
-	GetFileInfoData *gfi_data = callback_data;
-	GthFileList     *file_list;
-	GList           *scan, *chunk;
-	int              i;
+	AddListData *al_data = callback_data;
+	GthFileList *file_list;
+	GList       *scan, *chunk;
+	int          i;
 
-	if ((gfi_data == NULL) || gfi_data->destroyed)
+	if ((al_data == NULL) || al_data->destroyed)
 		return FALSE;
 
-	file_list = gfi_data->file_list;
+	file_list = al_data->file_list;
 
-	if (gfi_data->timeout_id != 0) {
-		g_source_remove (gfi_data->timeout_id);
-		gfi_data->timeout_id = 0;
+	if (al_data->timeout_id != 0) {
+		g_source_remove (al_data->timeout_id);
+		al_data->timeout_id = 0;
 	}
 
 	if (file_list->priv->finalizing || file_list->priv->stop_it) {
-		gfi_data->destroyed = TRUE;
+		al_data->destroyed = TRUE;
 		file_list->priv->load_thumbs = file_list->enable_thumbs;
 		if (! file_list->priv->finalizing)
-			g_idle_add (add_list_in_chunks_stopped, gfi_data);
+			g_idle_add (add_list_in_chunks_stopped, al_data);
 		return FALSE;
 	}
 
-	if (gfi_data->filtered == NULL) {
-		gfi_data->destroyed = TRUE;
-		get_file_info_data_free (gfi_data);
+	if (al_data->scan == NULL) {
+		al_data->destroyed = TRUE;
+		add_list_data_free (al_data);
 		add_list_done (file_list);
 		return FALSE;
 	}
@@ -1098,7 +1102,7 @@ add_list_in_chunks (gpointer callback_data)
 	gth_file_view_freeze (file_list->view);
 	gth_file_view_unsorted (file_list->view);
 
-	for (i = 0, scan = gfi_data->filtered; (i < ADD_LIST_CHUNK_SIZE) && scan; i++, scan = scan->next) {
+	for (i = 0, scan = al_data->scan; (i < ADD_LIST_CHUNK_SIZE) && scan; i++, scan = scan->next) {
 		FileData  *fd = scan->data;
 		GdkPixbuf *pixbuf;
 
@@ -1122,13 +1126,13 @@ add_list_in_chunks (gpointer callback_data)
 		scan->prev = NULL;
 	}
 
-	chunk = gfi_data->filtered;
-	gfi_data->filtered = scan;
-	file_list->list = g_list_concat (file_list->list, chunk);
+	chunk = al_data->scan;
+	al_data->scan = scan;
+	file_list->list = g_list_concat (file_list->list, file_data_list_dup (chunk));
 
-	gfi_data->timeout_id = g_timeout_add (ADD_LIST_DELAY,
-					      add_list_in_chunks,
-					      gfi_data);
+	al_data->timeout_id = g_timeout_add (ADD_LIST_DELAY,
+					     add_list_in_chunks,
+					     al_data);
 
 	return FALSE;
 }
@@ -1137,143 +1141,50 @@ add_list_in_chunks (gpointer callback_data)
 /* -- gth_file_list_set_list -- */
 
 
-static gboolean
-set_list_stopped (gpointer callback_data)
-{
-	GetFileInfoData *gfi_data = callback_data;
-	GthFileList     *file_list = gfi_data->file_list;
-
-	get_file_info_data_free (gfi_data);
-	gth_file_list_exec_next_op (file_list);
-
-	return FALSE;
-}
-
-
-static void
-set_list__get_file_info_done_cb (GnomeVFSAsyncHandle *handle,
-				 GList               *results, /* GnomeVFSGetFileInfoResult* items */
-				 gpointer             callback_data)
-{
-	GetFileInfoData *gfi_data = callback_data;
-	GthFileList     *file_list;
-	GList           *scan;
-
-	if ((gfi_data == NULL) || gfi_data->destroyed)
-		return;
-
-	file_list = gfi_data->file_list;
-	file_list->priv->get_file_list_handle = NULL;
-
-	if (file_list->priv->finalizing) {
-		get_file_info_data_free (gfi_data);
-		return;
-	}
-
-	g_signal_emit (G_OBJECT (file_list), gth_file_list_signals[IDLE], 0);
-
-	if (file_list->priv->stop_it) {
-		gfi_data->destroyed = TRUE;
-		g_idle_add (set_list_stopped, gfi_data);
-		return;
-	}
-
-	for (scan = results; scan; scan = scan->next) {
-		GnomeVFSGetFileInfoResult *info_result = scan->data;
-		char                      *uri;
-		FileData                  *fd;
-
-		if ((info_result->result != GNOME_VFS_OK) || (info_result->uri == NULL))
-			continue;
-
-		uri = gnome_vfs_uri_to_string (info_result->uri, GNOME_VFS_URI_HIDE_NONE);
-
-		fd = file_data_new (uri, info_result->file_info);
-		fd->mime_type = g_hash_table_lookup (gfi_data->mime_types, uri);
-
-		gfi_data->filtered = g_list_prepend (gfi_data->filtered, fd);
-
-		g_free (uri);
-	}
-
-	add_list_in_chunks (gfi_data);
-}
-
-
 static void
 load_new_list (GthFileList *file_list)
 {
-	GetFileInfoData *gfi_data;
-	GList           *scan;
-	gboolean         fast_file_type;
-
+	GList *scan;
+	
 	if (file_list->priv->stop_it)
 		return;
 
+	/* if an image is already present invalidate the thumbnail
+	 * instead of adding the image twice. */
+
+	for (scan = file_list->priv->new_list; scan; /* empty */) {
+		FileData *new_fd = scan->data;
+		GList    *same_path_in_list;
+		GList    *next = scan->next;
+		
+		same_path_in_list = file_data_list_find_path (file_list->list, new_fd->path); 
+		if (same_path_in_list != NULL) {
+			FileData *old_fd = same_path_in_list->data;
+			
+			old_fd->error = FALSE;
+			old_fd->thumb_loaded = FALSE;
+			old_fd->thumb_created = FALSE;
+			
+			/* remove the file from new_list */
+			
+			file_list->priv->new_list = g_list_remove_link (file_list->priv->new_list, scan);
+			file_data_list_free (scan);
+		}
+		
+		scan = next;
+	}
+
 	if (file_list->priv->new_list == NULL) {
 		add_list_done (file_list);
 		return;
 	}
 
-	gfi_data = get_file_info_data_new (file_list);
+	/**/
 
 	file_list->busy = TRUE;
 	file_list->priv->stop_it = FALSE;
-
 	g_signal_emit (G_OBJECT (file_list), gth_file_list_signals[BUSY], 0);
-
-	fast_file_type = eel_gconf_get_boolean (PREF_FAST_FILE_TYPE, TRUE);
-	for (scan = gfi_data->new_list; scan; scan = scan->next) {
-		char        *full_path = scan->data;
-		const char  *name_only = file_name_from_path (full_path);
-		const char  *mime_type;
-		GnomeVFSURI *uri;
-		FileData    *fd;
-
-		mime_type = get_file_mime_type (full_path, fast_file_type);
-		if (mime_type == NULL)
-			continue;
-
-		if ((! file_list->priv->show_dot_files && file_is_hidden (name_only))
-		    || ! (mime_type_is_image (mime_type) || 
-			  mime_type_is_video (mime_type) ||
-			  mime_type_is_audio (mime_type) ))
-			continue;
-
-		/* if the image is already present invalidate the thumbnail
-		 * instead of adding the image twice. */
-
-		fd = gth_file_list_filedata_from_path (file_list, full_path);
-		if (fd != NULL) {
-			fd->mime_type = mime_type;
-			fd->error = FALSE;
-			fd->thumb_loaded = FALSE;
-			fd->thumb_created = FALSE;
-			continue;
-		}
-
-		uri = new_uri_from_path (full_path);
-		if (uri != NULL) {
-			gfi_data->uri_list = g_list_prepend (gfi_data->uri_list, uri);
-			g_hash_table_insert (gfi_data->mime_types,
-					     full_path,
-					     (gpointer) mime_type);
-
-			/* do not free this path because its used in the mime_type hash. */
-			scan->data = NULL;
-		}
-	}
-
-	path_list_free (gfi_data->new_list);
-	gfi_data->new_list = NULL;
-
-	gnome_vfs_async_get_file_info (&file_list->priv->get_file_list_handle,
-				       gfi_data->uri_list,
-				       (GNOME_VFS_FILE_INFO_DEFAULT
-					| GNOME_VFS_FILE_INFO_FOLLOW_LINKS),
-				       GNOME_VFS_PRIORITY_MAX,
-				       set_list__get_file_info_done_cb,
-				       gfi_data);
+	add_list_in_chunks (add_list_data_new (file_list));
 }
 
 
@@ -1288,7 +1199,7 @@ gfl_set_list (GthFileList *file_list,
 	gth_file_list_free_list (file_list);
 
 	if (file_list->priv->new_list != new_list)
-		path_list_free (file_list->priv->new_list);
+		file_data_list_free (file_list->priv->new_list);
 	file_list->priv->new_list = new_list;
 
 	load_new_list (file_list);
@@ -1303,9 +1214,9 @@ gth_file_list_set_list (GthFileList   *file_list,
 {
 	file_list->priv->sort_method = sort_method;
 	file_list->priv->sort_type = sort_type;
-	gth_file_list_queue_op_with_list (file_list,
-					  GTH_FILE_LIST_OP_TYPE_SET_LIST,
-					  new_list);
+	gth_file_list_queue_op_with_file_data_list (file_list,
+					  	    GTH_FILE_LIST_OP_TYPE_SET_LIST,
+					  	    new_list);
 }
 
 
@@ -1319,6 +1230,7 @@ gfl_empty_list (GthFileList *file_list)
 		path_list_free (file_list->priv->new_list);
 		file_list->priv->new_list = NULL;
 	}
+	gth_file_view_clear (file_list->view);
 }
 
 
@@ -1346,9 +1258,9 @@ void
 gth_file_list_add_list (GthFileList *file_list,
 			GList       *new_list)
 {
-	gth_file_list_queue_op_with_list (file_list,
-					  GTH_FILE_LIST_OP_TYPE_ADD_LIST,
-					  new_list);
+	gth_file_list_queue_op_with_file_data_list (file_list,
+					  	    GTH_FILE_LIST_OP_TYPE_ADD_LIST,
+					  	    new_list);
 }
 
 
@@ -1432,7 +1344,7 @@ gth_file_list_pos_from_path (GthFileList *file_list,
 
 		i++;
 	}
-	g_list_free (list);
+	file_data_list_free (list);
 
 	return retval;
 }
@@ -1454,11 +1366,11 @@ gth_file_list_filedata_from_path (GthFileList *file_list,
 	for (scan = list; scan; scan = scan->next) {
 		FileData *fd = scan->data;
 		if (same_uri (fd->path, path)) {
-			result = fd;
+			result = file_data_ref (fd);
 			break;
 		}
 	}
-	g_list_free (list);
+	file_data_list_free (list);
 
 	return result;
 }
@@ -1494,7 +1406,7 @@ gth_file_list_get_all_from_view (GthFileList *file_list)
 		FileData *fd = scan->data;
 		path_list = g_list_prepend (path_list, g_strdup (fd->path));
 	}
-	g_list_free (list);
+	file_data_list_free (list);
 
 	return g_list_reverse (path_list);
 }
@@ -1707,10 +1619,10 @@ gfl_delete (GthFileList *file_list,
 	FileData *fd;
 
 	fd = gth_file_list_filedata_from_path (file_list, uri);
-	if (fd == NULL)
-		return;
-
-	gth_file_view_remove (file_list->view, fd);
+	if (fd != NULL) {
+		gth_file_view_remove (file_list->view, fd);
+		file_data_unref (fd);
+	}
 }
 
 
@@ -1829,20 +1741,22 @@ gfl_update_thumb (GthFileList *file_list,
 	FileData *fd;
 	int       pos;
 
-	fd = gth_file_list_filedata_from_path (file_list, uri);
-	if (fd == NULL)
-		return;
-
-	file_data_update (fd);
-
 	pos = gth_file_list_pos_from_path (file_list, uri);
 	if (pos == -1)
 		return;
 
+	fd = gth_file_list_filedata_from_path (file_list, uri);
+	if (fd == NULL)
+		return;
+
+	file_data_update_info (fd);
+
 	file_list->priv->thumb_pos = pos;
 	if (file_list->priv->thumb_fd != NULL)
 		file_data_unref (file_list->priv->thumb_fd);
-	file_list->priv->thumb_fd = fd;
+	file_list->priv->thumb_fd = file_data_ref (fd);
+	
+	file_data_unref (fd);
 }
 
 
@@ -1872,7 +1786,7 @@ gfl_update_thumb_list (GthFileList  *file_list,
 			continue;
 
 		fd = gth_file_view_get_image_data (file_list->view, pos);
-		file_data_update (fd);
+		file_data_update_info (fd);
 		file_data_unref (fd);
 	}
 }
@@ -2006,11 +1920,11 @@ gth_file_list_exec_next_op (GthFileList *file_list)
 		gfl_update_thumb_list (file_list, op->file.uri_list);
 		break;
 	case GTH_FILE_LIST_OP_TYPE_ADD_LIST:
-		gfl_add_list (file_list, op->file.uri_list);
+		gfl_add_list (file_list, op->file.fd_list);
 		exec_next_op = FALSE;
 		break;
 	case GTH_FILE_LIST_OP_TYPE_SET_LIST:
-		gfl_set_list (file_list, op->file.uri_list);
+		gfl_set_list (file_list, op->file.fd_list);
 		exec_next_op = FALSE;
 		break;
 	case GTH_FILE_LIST_OP_TYPE_DELETE_LIST:
@@ -2032,6 +1946,7 @@ gth_file_list_exec_next_op (GthFileList *file_list)
 		break;
 	}
 	op->file.uri_list = NULL;
+	op->file.fd_list = NULL;
 	gth_file_list_op_free (op);
 
 	if (exec_next_op)
