@@ -138,7 +138,7 @@ gth_window_activate_action_file_save (GtkAction *action,
 
 	gth_window_save_pixbuf (window,
 				image_viewer_get_current_pixbuf (image_viewer),
-				gth_window_get_image_filename (window));
+				gth_window_get_image_data (window));
 }
 
 
@@ -173,15 +173,12 @@ print_done_cb (gpointer data)
 
 	tmp_comment = comments_get_comment_filename (tmp_filename, TRUE);
 	tmp_dir = remove_level_from_path (tmp_comment);
-	file_unlink (tmp_comment);
-	dir_remove (tmp_dir);
+	local_dir_remove_recursive (tmp_dir);
 	g_free (tmp_comment);
 	g_free (tmp_dir);
 
-	file_unlink (tmp_filename);
-
 	tmp_dir = remove_level_from_path (tmp_filename);
-	dir_remove (tmp_dir);
+	local_dir_remove_recursive (tmp_dir);	
 	g_free (tmp_dir);
 
 	g_free (tmp_filename);
@@ -192,73 +189,77 @@ void
 gth_window_activate_action_file_print (GtkAction *action,
 				       GthWindow *window)
 {
-	GList    *list;
-	char     *tmp_dir;
-	char     *tmp_filename = NULL;
-	gboolean  remove_temp_file = FALSE;
+	GList *list;
 
-	list = gth_window_get_file_list_selection (window);
+	list = gth_window_get_file_list_selection_as_fd (window);
 	if (list == NULL)
 		return;
 
 	if (gth_window_get_image_modified (window)) {
+		char        *tmp_dir = NULL;
+		char        *tmp_filename = NULL;
 		ImageViewer *image_viewer;
 		GdkPixbuf   *pixbuf;
+		GError      *error = NULL;
+		const char  *image_filename;
+		GList       *current;
 
 		image_viewer = gth_window_get_image_viewer (window);
 		pixbuf = image_viewer_get_current_pixbuf (image_viewer);
-		if (pixbuf != NULL) {
-			GError     *error = NULL;
-			const char *image_filename;
-			GList      *current;
-
-			g_object_ref (pixbuf);
-			tmp_dir = get_temp_dir_name ();
-			if (tmp_dir == NULL)
-			{
-				_gtk_error_dialog_run (GTK_WINDOW (window), _("Could not create a temporary folder"));
-				return;
-			}
-			tmp_filename = get_temp_file_name (tmp_dir, ".jpeg");
-			if (! _gdk_pixbuf_save (pixbuf,
-						tmp_filename,
-						"jpeg",
-						&error,
-						NULL)) {
-				_gtk_error_dialog_from_gerror_run (GTK_WINDOW (window), &error);
-				g_object_unref (pixbuf);
-				g_free (tmp_filename);
-				dir_remove(tmp_dir);
-				g_free(tmp_dir);
-				return;
-			}
+		if (pixbuf == NULL) 
+			return;
+			
+		g_object_ref (pixbuf);
+		tmp_dir = get_temp_dir_name ();
+		if (tmp_dir == NULL) {
+			_gtk_error_dialog_run (GTK_WINDOW (window), _("Could not create a temporary folder"));
 
 			g_object_unref (pixbuf);
+			file_data_list_free (list);
 
-			image_filename = gth_window_get_image_filename (window);
-			current = g_list_find_custom (list,
-						      image_filename,
-						      (GCompareFunc) strcmp);
-			if (current != NULL) {
-				g_free (current->data);
-				current->data = g_strdup (tmp_filename);
-				comment_copy (image_filename, tmp_filename);
-			}
-
-			remove_temp_file = TRUE;
+			return;
 		}
-	}
+			
+		tmp_filename = get_temp_file_name (tmp_dir, ".jpeg");
+		if (! _gdk_pixbuf_save (pixbuf,
+					tmp_filename,
+					"jpeg",
+					&error,
+					NULL)) 
+		{
+			_gtk_error_dialog_from_gerror_run (GTK_WINDOW (window), &error);
+			local_dir_remove_recursive (tmp_dir);
 
-	if (remove_temp_file)
-	{
-		print_catalog_dlg_full (GTK_WINDOW (window), list, print_done_cb, tmp_filename);
-		dir_remove (tmp_dir);
+			g_object_unref (pixbuf);				
+			g_free (tmp_filename);
+			g_free (tmp_dir);
+			file_data_list_free (list);
+				
+			return;
+		}
+
+		g_object_unref (pixbuf);
+
+		image_filename = gth_window_get_image_filename (window);
+		current = file_data_list_find_path (list, image_filename);
+		if (current != NULL) {
+			FileData *fd;
+					
+			fd = file_data_dup (current->data);
+			g_free (fd->path);
+			fd->path =  get_uri_from_local_path (tmp_filename);
+			comment_copy (image_filename, fd->path);
+		}
+
+		print_catalog_dlg_full (GTK_WINDOW (window), list, print_done_cb, get_uri_from_local_path (tmp_filename));
+
+		g_free (tmp_filename);		
 		g_free (tmp_dir);
 	}
 	else
 		print_catalog_dlg (GTK_WINDOW (window), list);
 
-	path_list_free (list);
+	file_data_list_free (list);
 }
 
 
@@ -607,12 +608,21 @@ gth_window_activate_action_view_zoom_100 (GtkAction *action,
 }
 
 
+void
+gth_window_activate_action_view_zoom_fit_if_larger (GtkAction *action,
+					            gpointer   data)
+{
+	GthWindow   *window = GTH_WINDOW (data);
+	image_viewer_set_fit_mode (gth_window_get_image_viewer (window), GTH_FIT_SIZE_IF_LARGER);
+}
+
+
 void
 gth_window_activate_action_view_zoom_fit (GtkAction *action,
 					  gpointer   data)
 {
 	GthWindow   *window = GTH_WINDOW (data);
-	image_viewer_set_fit_mode (gth_window_get_image_viewer (window), GTH_FIT_SIZE_IF_LARGER);
+	image_viewer_set_fit_mode (gth_window_get_image_viewer (window), GTH_FIT_SIZE);
 }
 
 
@@ -632,25 +642,40 @@ gth_window_activate_action_view_exit_fullscreen (GtkAction *action,
 }
 
 
-static void
-set_wallpaper (const char     *image_path,
-	       WallpaperAlign  align)
+typedef struct {
+	GthWindow      *window;
+	WallpaperAlign  align;
+} SetWallpaperData;
+
+
+static void 
+set_wallpaper_step_2 (const char     *uri, 
+		      GnomeVFSResult  result, 
+		      gpointer        callback_data)
 {
-	GConfClient *client;
-	char        *options = "none";
+	SetWallpaperData *data = callback_data;
+	GConfClient      *client;
+	char             *options = "none";
+
+	if (result != GNOME_VFS_OK) {
+		_gtk_error_dialog_run (GTK_WINDOW (data->window), "%s", gnome_vfs_result_to_string (result));
+		g_free (data);
+		return;
+	}
 
 	client = gconf_client_get_default ();
 
-	image_path = get_file_path_from_uri (image_path);
-	if ((image_path == NULL) || ! path_is_file (image_path))
-		options = "none";
-
-	else {
+	if (path_is_file (uri)) {
+		char *image_path;
+		
+		image_path = get_local_path_from_uri (uri);
 		gconf_client_set_string (client,
 					 "/desktop/gnome/background/picture_filename",
 					 image_path,
 					 NULL);
-		switch (align) {
+		g_free (image_path);
+		
+		switch (data->align) {
 		case WALLPAPER_ALIGN_TILED:
 			options = "wallpaper";
 			break;
@@ -674,6 +699,8 @@ set_wallpaper (const char     *image_path,
 				 options,
 				 NULL);
         g_object_unref (G_OBJECT (client));
+        
+	g_free (data);	
 }
 
 
@@ -683,31 +710,75 @@ get_wallpaper_filename (int n)
 	char *name, *filename;
 
 	name = g_strdup_printf (".temp_wallpaper_%d.png", n);
-	filename = g_build_filename ("file://",
-				     g_get_home_dir (),
-				     name,
-				     NULL);
+	filename = g_strconcat ("file://",
+				g_get_home_dir (),
+				"/",
+				name,
+				NULL);
 	g_free (name);
 
 	return filename;
 }
 
 
+static char *
+get_new_wallpaper_filename (void) 
+{
+	char *wallpaper_filename;
+	
+	wallpaper_filename = get_wallpaper_filename (1);
+	if (path_is_file (wallpaper_filename)) {
+		/* Use a new filename to force an update. */
+		file_unlink (wallpaper_filename);
+		g_free (wallpaper_filename);
+		wallpaper_filename = get_wallpaper_filename (2);
+		if (path_is_file (wallpaper_filename))
+			file_unlink (wallpaper_filename);
+	}
+	
+	return 	wallpaper_filename;
+}
+
+
+static void
+set_wallpaper (GthWindow      *window,
+	       const char     *image_path,
+	       WallpaperAlign  align)
+{
+	SetWallpaperData *data;
+	
+	data = g_new0 (SetWallpaperData, 1);
+	data->window = window;
+	data->align = align;
+	
+	if (is_local_file (image_path))
+		set_wallpaper_step_2 (image_path, GNOME_VFS_OK, data);
+	else {
+		char *wallpaper_filename;
+		
+		wallpaper_filename = get_new_wallpaper_filename ();
+		copy_file_async (image_path, wallpaper_filename, set_wallpaper_step_2, data);
+		g_free (wallpaper_filename);
+	}
+}
+
+
 static void
 set_wallpaper_from_window (GthWindow      *window,
 			   WallpaperAlign  align)
 {
 	char *image_path = NULL;
 
-	if (!gth_window_get_image_modified (window)) {
+	if (! gth_window_get_image_modified (window)) {
 		const char *filename = gth_window_get_image_filename (window);
 		if (filename != NULL)
 			image_path = g_strdup (filename);
-
-	} else {
+	} 
+	else {
 		ImageViewer *image_viewer;
 		GdkPixbuf   *pixbuf;
 		char        *wallpaper_filename = NULL;
+		char        *local_file;
 		GError      *error = NULL;
 
 		image_viewer = gth_window_get_image_viewer (window);
@@ -717,33 +788,28 @@ set_wallpaper_from_window (GthWindow      *window,
 
 		g_object_ref (pixbuf);
 
-		wallpaper_filename = get_wallpaper_filename (1);
-		if (path_is_file (wallpaper_filename)) {
-			/* Use a new filename to force an update. */
-			file_unlink (wallpaper_filename);
-			g_free (wallpaper_filename);
-			wallpaper_filename = get_wallpaper_filename (2);
-			if (path_is_file (wallpaper_filename))
-				file_unlink (wallpaper_filename);
-		}
-
+		wallpaper_filename = get_new_wallpaper_filename ();
+		local_file = get_local_path_from_uri (wallpaper_filename);
 		if (! _gdk_pixbuf_save (pixbuf,
-					wallpaper_filename,
+					local_file,
 					"jpeg",
 					&error,
-					NULL)) {
+					NULL)) 
+		{
 			_gtk_error_dialog_from_gerror_run (GTK_WINDOW (window), &error);
 			g_object_unref (pixbuf);
+			g_free (local_file);			
 			g_free (wallpaper_filename);
 			return;
 		}
 
-		g_object_unref (pixbuf);
-
 		image_path = wallpaper_filename;
+		
+		g_object_unref (pixbuf);
+		g_free (local_file);
 	}
 
-	set_wallpaper (image_path, align);
+	set_wallpaper (window, image_path, align);
 	g_free (image_path);
 }
 
@@ -788,7 +854,8 @@ void
 gth_window_activate_action_wallpaper_restore (GtkAction *action,
 					      gpointer   data)
 {
-	int align_type = WALLPAPER_ALIGN_CENTERED;
+	GthWindow *window = GTH_WINDOW (data);
+	int        align_type = WALLPAPER_ALIGN_CENTERED;
 
 	if (strcmp (preferences.wallpaperAlign, "wallpaper") == 0)
 		align_type = WALLPAPER_ALIGN_TILED;
@@ -801,7 +868,7 @@ gth_window_activate_action_wallpaper_restore (GtkAction *action,
 	else if (strcmp (preferences.wallpaperAlign, "none") == 0)
 		align_type = WALLPAPER_NONE;
 
-	set_wallpaper (preferences.wallpaper, align_type);
+	set_wallpaper (window, preferences.wallpaper, align_type);
 }
 
 
@@ -897,7 +964,7 @@ gth_window_activate_action_tools_change_date (GtkAction *action,
 
 void
 gth_window_activate_action_tools_reset_exif (GtkAction *action,
-                                              GthWindow *window)
+                                             GthWindow *window)
 {
         dlg_reset_exif (window);
 }
